<?php
/**
 * Class for displaying and editing classification
 * 
 * @author Tomas Kadlec <tomas@tomaskadlec.net>
 * @package
 * @subpackage
 * @uses ClassificationStructure, ClassificationCommonException and ancestors
 */
class Classification {
		
	/** study type */
	private $study;
	
	/** class number */
	private $class;
	
	/** array with students */
	private $students;

	/**
	 * Constructor
	 * @param $study - study type - 0 daily, 1 - distance
	 * @param $class - class number (1 - 999)
	 * @param $ser - serialized class data (can be null, to create an empty class)
	 * @throws ClassificationStudyTypeException, ClassicicationInitializationFailedException
	 */
	public function __construct($study, $class, $ser = null) {
		// check && set study type
		if ($study != 0 && $study != 1) throw new ClassificationStudyTypeException();
		$this->study = $study;	
		// check && set class and instance mode
		if (!is_numeric($class) || $class <= 0 || $class >= 10000)
			throw new ClassificationClassOutOfRangeException();
		$this->class = $class;
		// unserialize data
		if ($ser !== null && $ser !== '') {
			$this->students = @unserialize($ser);
			if ($this->students === false)
				$this->students = array();
				//throw new ClassificationInitializationFailedException();
			
		}
		// or create an empty class
		else
			$this->students = array();
	}
	
	/**
	 * Cheks if a student is enrolled in this particular class.
	 * @param $login - student's login
	 * @return true or false
	 */
	public function isStudent($login) {
		if (isset($this->students[$login]))
			return true;
		else
			return false;
	}
	
	/**
	 * Data for output
	 * @param $login - shows only row according to login
	 * @return classification array according to defined structure (deep copy)
	 * @throws ClassificationLoginNotFoundException
	 */
	public function output($login = null) {
		if ($login !== null) {
			 if (!isset($this->students[$login])) throw new ClassificationLoginNotFoundException();
			 return $this->students[$login];
		}
		else {
			return $this->students;
		}
	}
	
	/**
	 * Import new student
	 * @param $login - login (sunray)
	 * @param $email - e-mail address
	 * @param $name - student's name
	 * @throws ClassificationUndefinedDataTypeException
	 */
	public function import($login, $email, $name) {
		// available types
		$t = &ClassificationStructure::$structure['types'];
		// defined structure
		$s = &ClassificationStructure::$structure['structure'];
		$this->students[$login] = array (
			'__info' => array($login, $email, $name, $this->class),
		);
		foreach ($s as $pos => $column) {
			switch ($column[2]) {
				// internal value - student
				case 0:
					if (ereg('student',$column[3]))
						$this->students[$login][$column[0]] = "<a class=\"mail\" href=\"mailto:$email\">$name</a>";
					break;
				// number, boolean, string - put default value
				case 1:
				case 2:
				case 3:
					$this->students[$login][$column[0]] = $column[3];
					break;
				// expression
				case 4:
					$this->students[$login][$column[0]] = '';
					break;
				// link
				case 5:
					$this->students[$login][$column[0]] = "[[:student:$login:start| $login]]";
					break;
				// error
				default:
					throw new ClassificationUndefinedDataTypeException();	
			}
		}
	}
	
	/**
	 * ksort students
	 */
	public function ksort() {
		@ksort($this->students);
	}
	
	/**
	 * Export class data into serialized form
	 * @return serialized array with classification
	 * @throws ClassificationSerializationException 
	 */
	public function export() {
		if (!is_array($this->students) || count($this->students) <= 0)
			return '';
		if (($ser = serialize($this->students)) !== false)
			return $ser;
		else
			throw new ClassificationSerializationException();
	}
	
	/**
	 * Transfer studens between classes
	 * @param $login - student's login
	 * @param $class - destination class
	 */
	public function transfer($login, Classification &$class) {
		$class->students[] = $this->students[$login];
		unset($this->students[$login]);
		// stored, collon delimited array of transfered students (for handlin imports)
		if (!isset($this->students['_transfered']))
			$this->students['_transfered'] .= "$login";
		else
			$this->students['_transfered'] .= ":$login";
	}
	
	/**
	 * Update  score
	 * @param $data - array with login => cols to update
	 * @throws ClassificationScoreUpdateFailedException
	 */
	public function update(array &$data) {
		$s = &ClassificationStructure::$structure['structure'];
		foreach($data as $login => &$cols) {
			if ($this->isStudent($login)) {
				foreach ($s as $column) {
					switch ($column[2]) {
						// number
						case 1:
							if (isset($data[$login][$column[0]]) && is_numeric($data[$login][$column[0]]))
								$this->students[$login][$column[0]] = $data[$login][$column[0]];
							break;
						// boolean
						case 2:
							if (isset($data[$login][$column[0]]))
								$this->students[$login][$column[0]] = 1;
							else
								$this->students[$login][$column[0]] = 0;
							break;
						// string
						case 3:
							if (isset($data[$login][$column[0]]) && is_string($data[$login][$column[0]]))
								$this->students[$login][$column[0]] = $data[$login][$column[0]];
							break;
						// expression
						case 4:
							//echo "$column[4]<br/>";
							$cmd = preg_replace('/--LOGIN--/', $login, $column[4]);
							$tmp = @eval($cmd);
							// if eval failed, let the column be "empty"
							if ($tmp === false) {
								$this->students[$login][$column[0]] = '';
							}
						// other types are not processed here
						default:
							break;
					}
				}
			}
		}
		//echo '<pre>'.print_r($this->students,true).'</pre>';
	}
	
} // Classification class end

/**
 * Common classification exception
 */
class ClassificationCommonException extends Exception {
	public function __construct($message = 'exCommon') {
		$this->message = $message;
	}
}
/**
 * Exception used when wrong study type is requested 
 */
class ClassificationStudyTypeException extends ClassificationCommonException {
	public function __construct($message = 'exClassificationStudyType') {
		$this->message = $message;
	}
}
/**
 * Exception used when ns does not exist or is not writable 
 */ 
class ClassificationInitializationFailedException extends ClassificationCommonException {
	public function __construct($message = 'exInitializationFailed') {
		$this->message = $message;
	}
}
/**
 * Exception used when requested class does not meet class number requirements 
 */
class ClassificationClassOutOfRangeException extends ClassificationCommonException {
	public function __construct($message = 'exClassOutOfRange') {
		$this->message = $message;
	}
}
/**
 * Exception used when data type does not exist 
 */
class ClassificationUndefinedDataTypeException extends ClassificationCommonException {
	public function __construct($message = 'exUndefinedDataType') {
		$this->message = $message;
	}
}
/**
 * Exception used when requesed login does not exist 
 */
class ClassificationLoginNotFoundException extends ClassificationCommonException {
	public function __construct($message = 'exLoginNotFound') {
		$this->message = $message;
	}
}
/**
 * Exception raised when serialization fails 
 */
class ClassificationSerializationException extends ClassificationCommonException {
	public function __construct($message = 'exSerialization') {
		$this->message = $message;
	}
}
?>